@@ -283,38 +283,54 @@ where
     let mut files: Vec<(String, PathBuf, u64, i64, u32)> = Vec::new(); // (rel_path, abs_path, size, mtime, mode)
     let mut symlinks: Vec<SymlinkEntry> = Vec::new();
 
-    for entry in entries {
-        if let Ok(rel_path) = entry.path.strip_prefix(root_path) {
-            if rel_path.as_os_str().is_empty() {
-                continue; // Skip root
-            }
-            if let Some(path_str) = rel_path.to_str() {
-                let mtime = entry
-                    .modified
-                    .duration_since(std::time::SystemTime::UNIX_EPOCH)
-                    .unwrap_or_default()
-                    .as_secs() as i64;
+    // Check if root_path is a single file (not a directory)
+    let is_single_file = root_path.is_file();
 
-                if entry.is_dir {
-                    directories.push(path_str.to_string());
-                } else if entry.is_symlink {
-                    if let Some(target) = entry.symlink_target {
-                        if let Some(target_str) = target.to_str() {
-                            symlinks.push(SymlinkEntry {
-                                path: path_str.to_string(),
-                                target: target_str.to_string(),
-                            });
-                        }
+    for entry in entries {
+        // For single file sources, don't strip prefix - use just the filename
+        let rel_path_str = if is_single_file {
+            // Single file: use just the filename as the relative path
+            root_path
+                .file_name()
+                .and_then(|n| n.to_str())
+                .map(|s| s.to_string())
+        } else {
+            // Directory: strip the root prefix to get relative path
+            entry
+                .path
+                .strip_prefix(root_path)
+                .ok()
+                .filter(|p| !p.as_os_str().is_empty()) // Skip root directory entry
+                .and_then(|p| p.to_str())
+                .map(|s| s.to_string())
+        };
+
+        if let Some(path_str) = rel_path_str {
+            let mtime = entry
+                .modified
+                .duration_since(std::time::SystemTime::UNIX_EPOCH)
+                .unwrap_or_default()
+                .as_secs() as i64;
+
+            if entry.is_dir {
+                directories.push(path_str.to_string());
+            } else if entry.is_symlink {
+                if let Some(target) = entry.symlink_target {
+                    if let Some(target_str) = target.to_str() {
+                        symlinks.push(SymlinkEntry {
+                            path: path_str.to_string(),
+                            target: target_str.to_string(),
+                        });
                     }
-                } else {
-                    files.push((
-                        path_str.to_string(),
-                        entry.path.to_path_buf(),
-                        entry.size,
-                        mtime,
-                        0o644,
-                    ));
                 }
+            } else {
+                files.push((
+                    path_str.to_string(),
+                    entry.path.to_path_buf(),
+                    entry.size,
+                    mtime,
+                    0o644,
+                ));
             }
         }
     }
@@ -367,7 +383,9 @@ where
     }
     let ack = protocol::FileListAck::read(stdin).await?;
 
-    // Step 3: Send files that client requested
+    // Step 3: Send files that client requested (pipelined - send all, then collect ACKs)
+    let mut files_sent: Vec<u32> = Vec::new();
+
     for decision in &ack.decisions {
         if decision.action == Action::Skip {
             continue;
@@ -394,7 +412,7 @@ where
             }
         };
 
-        // Send FILE_DATA
+        // Send FILE_DATA (pipelined - no flush/wait per file)
         let file_data = FileData {
             index: decision.index,
             offset: 0,
@@ -402,9 +420,14 @@ where
             data,
         };
         file_data.write(stdout).await?;
-        stdout.flush().await?;
+        files_sent.push(decision.index);
+    }
 
-        // Wait for FILE_DONE
+    // Flush once after sending all files
+    stdout.flush().await?;
+
+    // Collect all FILE_DONE responses
+    for _ in &files_sent {
         let _len = stdin.read_u32().await?;
         let type_byte = stdin.read_u8().await?;
         if type_byte != MessageType::FileDone as u8 {
