@@ -44,6 +44,10 @@ impl ServerSession {
             cmd.arg("-i").arg(key);
         }
 
+        // Non-interactive mode options
+        cmd.arg("-o").arg("StrictHostKeyChecking=no");
+        cmd.arg("-o").arg("UserKnownHostsFile=/dev/null");
+
         // Remote command: sy --server <remote_path>
         cmd.arg("sy");
         cmd.arg("--server");
@@ -377,6 +381,10 @@ impl ServerSession {
             cmd.arg("-i").arg(key);
         }
 
+        // Non-interactive mode options
+        cmd.arg("-o").arg("StrictHostKeyChecking=no");
+        cmd.arg("-o").arg("UserKnownHostsFile=/dev/null");
+
         cmd.arg("sy");
         cmd.arg("--server");
         cmd.arg(remote_path);
